#include <avr/io.h> 
#include <avr/interrupt.h> 
#include <inttypes.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <util/delay.h> 
//#include <hd44780.c> 

//---------/ Defining ports /-------------- 

#define SDA 0 
#define SCL 1 

#define SDA_DDR DDRD 
#define SCL_DDR DDRD 

#define SCL_PIN PD6 
#define SDA_PIN PD7 

#define SDA_PORT PORTD 
#define SCL_PORT PORTD 

//----/ Macros for setting ports as IO /-------- 

#define SDA_OUT SDA_DDR |= (1 << SDA)        // SDA as output 
#define SDA_IN  SDA_DDR &= ~(1 << SDA)       // SDA as input 

#define SCL_OUT SCL_DDR |= (1 << SCL)        // SCL as output 
#define SCL_IN  SCL_DDR &= ~(1 << SCL)       // SCL as input 

//---/ Macros for sending signal over SDA & SCL /---- 

#define SET_SDA SDA_PORT |= (1<<SDA) //set SDA 
#define CLR_SDA SDA_PORT &= ~(1<<SDA) //clear SDA 

#define SET_SCL     SCL_PORT |= (1<<SCL) //set SCL 
#define CLR_SCL    SCL_PORT &= ~(1<<SCL) //clear SCL 


#define GET_SDA (SDA_PIN & (1 << SDA)) //for checking state of SDA 

#define ACK 0 
#define NOACK 1 

//--------------------------------------------------- 

void delay(void)          //just a little delay 
{ 

asm("nop"); 
asm("nop"); 

} 

//------------/I2C start sequence/----------------- 
  
void i2c_start(void) 
{ 

  SDA_OUT; 
  SCL_OUT; 
  SET_SDA;            
  SET_SCL; 
  delay(); 
  CLR_SDA; 
  delay(); 
  CLR_SCL; 

} 

//------------/ I2C stop sequence/----------------- 

void i2c_stop(void) 
{ 

  CLR_SDA;            
  delay(); 
  SET_SCL; 
  delay(); 
  SET_SDA; 
  delay(); 

} 

//--/ Function sending a byte over I2C /--- 

char i2c_write(unsigned char x) 
{ 

   unsigned char ack; 

   SDA_OUT; 

   for (char i=0;i<8;i++) 
   { 
      CLR_SCL; 
      if(x&0x80)          //check if the oldest bit is 0 or 1 
      { 
         SET_SDA;         //if 1 send 1 to SDA 
         delay(); 
         SET_SCL; 
         delay(); 
         CLR_SCL; 
         delay(); 
         CLR_SDA; 
      } 
      else 
      { 
         SET_SCL;         //if 0 send 0 to SDA 
         delay(); 
         CLR_SCL; 
         delay(); 
      } 
      x = x<<1; 
   } 

   SDA_IN;                //release the SDA 
   delay(); 
   SET_SCL; 
   delay(); 
   ack = GET_SDA;         //check for ACK 
   CLR_SCL; 

   if(ack)                // If not acknowledged return 1 
   { 
      return 1; 
   } 
    
   return 0; //If acknowledged return 0 
} 

//-----/ Function reading a byte + ACK over I2C/----- 

unsigned char i2c_read() 
{ 
   unsigned char count = 8, bufor = 0; 

   SDA_IN;                                  
   SET_SDA;  
                                
   do 
   { 
      bufor = bufor << 1;                                                    
      SET_SCL;                                    
      delay();                                    
      if(GET_SDA) bufor++;                        
      CLR_SCL;                                    
   } while (--count); 

   delay();                                
   SDA_OUT; 
   CLR_SDA; 
   SET_SCL;                                      
   delay(); 
   CLR_SCL; 
   return (bufor);        //returns the read byte            
} 
